import 'package:flutter/material.dart';

enum AnimationType { normal, fadeIn }

class RouteTransition {
  final BuildContext? context;
  final Widget? child;
  final AnimationType animation;
  final Duration duration;
  final bool replacement;

  RouteTransition({
    @required this.context,
    @required this.child,
    this.replacement = false,
    this.animation = AnimationType.normal,
    this.duration = const Duration(milliseconds: 300),
  }) {
    switch (this.animation) {
      case AnimationType.normal:
        this._normalTransition();
        break;
      case AnimationType.fadeIn:
        this._fadeInTransition();
        break;
    }
  }

  void _pushPage(Route route) => Navigator.push(this.context!, route);

  void _pushReplacement(Route route) =>
      Navigator.pushReplacement(this.context!, route);

  void _normalTransition() {
    final route = MaterialPageRoute(builder: (_) => this.child!);
    (this.replacement) ? this._pushReplacement(route) : this._pushPage(route);
  }

  void _fadeInTransition() {
    final route = PageRouteBuilder(
        pageBuilder: (_, __, ___) => this.child!,
        transitionDuration: this.duration,
        transitionsBuilder: (_, animation, __, child) {
          return FadeTransition(
            child: child,
            opacity: Tween<double>(begin: 0.0, end: 1.0).animate(
              CurvedAnimation(parent: animation, curve: Curves.easeOut),
            ),
          );
        });

    (this.replacement) ? this._pushReplacement(route) : this._pushPage(route);
  }
}
